package data_stat

import (
	"eta_gn/eta_index_lib/global"
)

// EdbInfoDeleteLog 指标删除日志表
type EdbInfoDeleteLog struct {
	Id                 uint64  `gorm:"primaryKey;autoIncrement;column:id"             description:"记录ID"`
	EdbInfoId          int     `gorm:"column:edb_info_id"                           description:"指标ID"`
	EdbInfoType        int     `gorm:"column:edb_info_type"                         description:"指标类型，0：普通指标，1：预测指标"`
	SourceName         string  `gorm:"column:source_name"                           description:"来源名称"`
	Source             int     `gorm:"column:source"                               description:"来源id"`
	EdbCode            string  `gorm:"column:edb_code"                             description:"指标编码"`
	EdbName            string  `gorm:"column:edb_name"                             description:"指标名称"`
	EdbNameEn          string  `gorm:"column:edb_name_en"                          description:"英文指标名称"`
	EdbNameSource      string  `gorm:"column:edb_name_source"                      description:"指标名称来源"`
	Frequency          string  `gorm:"column:frequency"                           description:"频率"`
	Unit               string  `gorm:"column:unit"                                description:"单位"`
	UnitEn             string  `gorm:"column:unit_en"                             description:"英文单位"`
	StartDate          string  `gorm:"column:start_date"                          description:"起始日期"`
	EndDate            string  `gorm:"column:end_date"                            description:"终止日期"`
	SysUserId          int     `gorm:"column:sys_user_id"                         description:"创建人ID"`
	SysUserRealName    string  `gorm:"column:sys_user_real_name"                  description:"创建人姓名"`
	UniqueCode         string  `gorm:"column:unique_code"                         description:"指标唯一编码"`
	EdbCreateTime      string  `gorm:"column:edb_create_time"                     description:"指标创建时间"`
	EdbModifyTime      string  `gorm:"column:edb_modify_time"                     description:"指标修改时间"`
	CreateTime         string  `gorm:"column:create_time"                         description:"创建时间即删除时间"`
	MinValue           float64 `gorm:"column:min_value"                           description:"指标最小值"`
	MaxValue           float64 `gorm:"column:max_value"                           description:"指标最大值"`
	CalculateFormula   string  `gorm:"column:calculate_formula"                   description:"计算公式"`
	EdbType            int     `gorm:"column:edb_type"                            description:"指标类型:1:基础指标,2:计算指标"`
	LatestDate         string  `gorm:"column:latest_date"                         description:"数据最新日期"`
	LatestValue        float64 `gorm:"column:latest_value"                        description:"数据最新值"`
	MoveType           int     `gorm:"column:move_type"                           description:"移动方式:1:领先(默认),2:滞后"`
	MoveFrequency      string  `gorm:"column:move_frequency"                      description:"移动频度"`
	NoUpdate           int8    `gorm:"column:no_update"                           description:"是否停止更新，0：继续更新；1：停止更新"`
	ChartImage         string  `gorm:"column:chart_image"                         description:"图表图片"`
	Calendar           string  `gorm:"column:calendar;default:公历"               description:"公历/农历"`
	DataDateType       string  `gorm:"column:data_date_type;size:255;null;default:交易日" description:"数据日期类型"`
	ManualSave         int     `gorm:"column:manual_save"                         description:"是否有手动保存过上下限: 0-否; 1-是"`
	TerminalCode       string  `gorm:"column:terminal_code"                       description:"终端编码，用于配置在机器上"`
	DelSysUserId       int     `gorm:"column:del_sys_user_id"                     description:"删除人ID"`
	DelSysUserRealName string  `gorm:"column:del_sys_user_real_name"              description:"删除人姓名"`
	DataUpdateTime     string  `gorm:"column:data_update_time"                    description:"最近一次数据发生变化的时间"`
	ErDataUpdateDate   string  `gorm:"column:er_data_update_date"                 description:"本次更新，数据发生变化的最早日期"`
}

type EdbInfoDeleteLogNum struct {
	Source       int    `description:"来源id"`
	TerminalCode string `description:"终端编码，用于配置在机器上"`
	Num          int    `description:"被删除的指标数据总数"`
}

func GetEdbDeleteLogNumByCreateTime(startDate, endDate string) (item []*EdbInfoDeleteLogNum, err error) {
	sql := ` SELECT source, terminal_code, count(*) as num FROM edb_info_delete_log WHERE create_time >= ? and create_time < ? group by source, terminal_code `
	err = global.DEFAULT_DmSQL.Raw(sql, startDate, endDate).Scan(&item).Error
	return
}
