package models

import (
	"errors"
	"eta_gn/eta_index_lib/global"
	"eta_gn/eta_index_lib/utils"
	"fmt"
	"github.com/shopspring/decimal"
	"gorm.io/gorm"
	"strconv"
	"strings"
	"time"
)

// SavePredictCalculateBp 变频
func SavePredictCalculateBp(req *EdbInfoCalculateBatchSaveReq, fromEdbInfo *EdbInfo, edbCode, uniqueCode string, sysUserId int, sysUserRealName, lang string) (edbInfo *EdbInfo, latestDateStr string, latestValue float64, err error) {
	to := global.DEFAULT_DmSQL.Begin()
	defer func() {
		if err != nil {
			to.Rollback()
		} else {
			to.Commit()
		}
	}()
	if req.EdbInfoId <= 0 {
		edbInfo = new(EdbInfo)
		edbInfo.EdbInfoType = 1
		edbInfo.Source = utils.DATA_SOURCE_PREDICT_CALCULATE_BP
		edbInfo.SourceName = "预测变频"
		edbInfo.EdbCode = edbCode
		edbInfo.EdbName = req.EdbName
		edbInfo.EdbNameSource = req.EdbName
		edbInfo.Frequency = req.Frequency
		edbInfo.Unit = req.Unit
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.SysUserId = sysUserId
		edbInfo.SysUserRealName = sysUserRealName
		edbInfo.CreateTime = time.Now()
		edbInfo.ModifyTime = time.Now()
		edbInfo.UniqueCode = uniqueCode
		edbInfo.EdbNameEn = req.EdbName
		edbInfo.UnitEn = req.Unit
		edbInfo.CalculateFormula = req.Formula
		edbInfo.EdbType = 2
		edbInfo.EmptyType = req.EmptyType
		edbInfo.Sort = GetAddEdbMaxSortByClassifyId(req.ClassifyId, utils.PREDICT_EDB_INFO_TYPE)
		tmpErr := to.Create(edbInfo).Error
		if tmpErr != nil {
			err = tmpErr
			return
		}
		//关联关系
		{
			calculateMappingItem := new(EdbInfoCalculateMapping)
			calculateMappingItem.CreateTime = time.Now()
			calculateMappingItem.ModifyTime = time.Now()
			calculateMappingItem.Sort = 1
			calculateMappingItem.EdbCode = edbCode
			calculateMappingItem.EdbInfoId = edbInfo.EdbInfoId
			calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
			calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
			calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
			calculateMappingItem.FromSource = fromEdbInfo.Source
			calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
			calculateMappingItem.FromTag = ""
			calculateMappingItem.Source = edbInfo.Source
			calculateMappingItem.SourceName = edbInfo.SourceName
			err = to.Create(calculateMappingItem).Error
			if err != nil {
				return
			}
		}
	} else {
		edbInfo, err = GetEdbInfoById(req.EdbInfoId)
		if err != nil {
			return
		}
		latestDateStr = edbInfo.LatestDate
		latestValue = edbInfo.LatestValue
		//修改指标信息
		switch lang {
		case utils.EnLangVersion:
			edbInfo.EdbNameEn = req.EdbName
			edbInfo.UnitEn = req.Unit
		default:
			edbInfo.EdbName = req.EdbName
			edbInfo.Unit = req.Unit
			edbInfo.EdbNameSource = req.EdbName
		}
		oldEmptyType := edbInfo.EmptyType
		edbInfo.Frequency = req.Frequency
		edbInfo.ClassifyId = req.ClassifyId
		edbInfo.EmptyType = req.EmptyType
		edbInfo.ModifyTime = time.Now()
		err = to.Model(edbInfo).Select([]string{"EdbName", "EdbNameSource", "Frequency", "Unit", "ClassifyId", "ModifyTime", "EdbNameEn", "UnitEn", "EmptyType"}).Updates(edbInfo).Error
		if err != nil {
			return
		}

		//判断计算指标是否被更换
		var existCondition string
		var existPars []interface{}
		existCondition += " AND edb_info_id=? AND from_edb_info_id=? "
		existPars = append(existPars, edbInfo.EdbInfoId, req.FromEdbInfoId)

		count, e := GetEdbInfoCalculateCountByCondition(existCondition, existPars)
		if e != nil {
			err = errors.New("判断指标是否改变失败,Err:" + e.Error())
			return
		}
		if count > 0 && oldEmptyType == edbInfo.EmptyType { // 指标未被替换，无需重新计算
			return
		}

		//删除，计算指标关联的，基础指标的关联关系
		sql := ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
		err = to.Exec(sql, edbInfo.EdbInfoId).Error
		if err != nil {
			err = errors.New("删除计算指标关联关系失败,Err:" + err.Error())
			return
		}
		//清空原有数据
		sql = ` DELETE FROM edb_data_predict_calculate_bp WHERE edb_info_id = ? `
		err = to.Exec(sql, edbInfo.EdbInfoId).Error
		if err != nil {
			return
		}
		//关联关系
		{
			calculateMappingItem := &EdbInfoCalculateMapping{
				EdbInfoCalculateMappingId: 0,
				EdbInfoId:                 edbInfo.EdbInfoId,
				Source:                    utils.DATA_SOURCE_PREDICT_CALCULATE_BP,
				SourceName:                "预测变频",
				EdbCode:                   edbInfo.EdbCode,
				FromEdbInfoId:             fromEdbInfo.EdbInfoId,
				FromEdbCode:               fromEdbInfo.EdbCode,
				FromEdbName:               fromEdbInfo.EdbName,
				FromSource:                fromEdbInfo.Source,
				FromSourceName:            fromEdbInfo.SourceName,
				FromTag:                   "",
				Sort:                      1,
				CreateTime:                time.Now(),
				ModifyTime:                time.Now(),
			}
			err = to.Create(calculateMappingItem).Error
			if err != nil {
				return
			}
		}
	}

	//计算数据
	latestDateStr, latestValue, err = refreshAllPredictCalculateBp(to, edbInfo.EdbInfoId, edbInfo.Source, edbInfo.SubSource, fromEdbInfo, edbInfo.EdbCode, "", "", 0, edbInfo.EmptyType)

	return
}

func RefreshAllPredictCalculateBp(edbInfoId, source, subSource int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string, emptyType int) (latestDateStr string, latestValue float64, err error) {
	to := global.DEFAULT_DmSQL.Begin()
	defer func() {
		if err != nil {
			to.Rollback()
		} else {
			to.Commit()
		}
	}()

	// 计算数据
	latestDateStr, latestValue, err = refreshAllPredictCalculateBp(to, edbInfoId, source, subSource, fromEdbInfo, edbCode, startDate, endDate, 0, emptyType)
	return
}

// refreshAllPredictCalculateBp 刷新变频数据
func refreshAllPredictCalculateBp(to *gorm.DB, edbInfoId, source, subSource int, fromEdbInfo *EdbInfo, edbCode, startDate, endDate string, order, emptyType int) (latestDateStr string, latestValue float64, err error) {
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据

	//获取来源指标的数据
	dataList, err := GetPredictEdbDataListAllByStartDate(fromEdbInfo, order, "")
	if err != nil {
		return
	}
	latestDateStr = fromEdbInfo.LatestDate

	var dateArr []string
	dataMap := make(map[string]*EdbInfoSearchData)
	fromDataMap := make(map[string]float64)
	//来源指指标数据
	for _, v := range dataList {
		dateArr = append(dateArr, v.DataTime)
		dataMap[v.DataTime] = v
		fromDataMap[v.DataTime] = v.Value
	}
	fmt.Println("source:", source)

	//获取升频指标所有数据
	existDataList, err := GetAllEdbDataListByTo(to, edbInfoId, source, subSource)
	if err != nil {
		return
	}
	//计算指标的map
	existDataMap := make(map[string]*EdbData, 0)
	removeDateMap := make(map[string]struct{})
	addSql := ` INSERT INTO edb_data_predict_calculate_bp(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `

	var isAdd bool
	//待删除的日期
	removeDateList := make([]string, 0)
	if len(existDataList) > 0 && len(dateArr) == 0 {
		//如果没有来源指标数据，那么已经入库的计算指标数据需要全部删除
		tableName := GetEdbDataTableName(source, subSource)
		sql := fmt.Sprintf(` DELETE FROM %s WHERE edb_info_id = ?`, tableName)
		err = to.Exec(sql, edbInfoId).Error
		if err != nil {
			err = fmt.Errorf("删除所有的升频指标数据失败,Err:" + err.Error())
			return
		}
		return
	}

	existMap := make(map[string]string)

	dataLen := len(dataList)
	//第三步： 已经入库的数据处理
	for _, v := range existDataList {
		existDataMap[v.DataTime] = v
		removeDateMap[v.DataTime] = struct{}{}
	}
	for i := 0; i < dataLen; i++ {
		//当期
		currentItem := dataList[i]
		var prevItem *EdbInfoSearchData
		if emptyType == 3 { //3后值填充,其余前值填充
			if i >= 1 {
				prevItem = dataList[i-1]
			}
		}
		currentDate, _ := time.ParseInLocation(utils.FormatDate, currentItem.DataTime, time.Local)
		var day int
		var preItem *EdbInfoSearchData
		var preDate time.Time
		if i == 0 {
			if emptyType == 3 { //后值填充
				day = 0 //最新的时间就是来源指标的最新日期
				preDate = currentDate
			} else {
				day = int(time.Now().Sub(currentDate).Hours() / float64(24))
				preDate = time.Date(time.Now().Year(), time.Now().Month(), time.Now().Day(), 0, 0, 0, 0, time.Local)
			}
		} else {
			j := i - 1
			if j < dataLen {
				preItem = dataList[j]
				preDate, _ = time.ParseInLocation(utils.FormatDate, preItem.DataTime, time.Local)
				day = int(preDate.Sub(currentDate).Hours() / float64(24))
				utils.FileLog.Info("preItem.DataTime:" + preItem.DataTime + ";currentItem.DataTime" + currentItem.DataTime)
			}
		}
		for k := 0; k <= day; k++ {
			needDay := preDate.AddDate(0, 0, -k)
			needDayStr := needDay.Format(utils.FormatDate)
			delete(removeDateMap, needDayStr)
			existKey := edbCode + needDayStr
			if _, ok := existMap[existKey]; !ok {
				timestamp := needDay.UnixNano() / 1e6
				timestampStr := fmt.Sprintf("%d", timestamp)
				valStr := decimal.NewFromFloat(currentItem.Value).String()
				if prevItem != nil && needDayStr != currentItem.DataTime {
					valStr = decimal.NewFromFloat(prevItem.Value).String()
				}
				if fromEdbInfo.LatestDate == needDayStr {
					if prevItem != nil && needDayStr != currentItem.DataTime {
						latestValue = prevItem.Value
					} else {
						latestValue = currentItem.Value
					}
				}

				tmpExistData, ok2 := existDataMap[needDayStr]
				if !ok2 {
					addSql += GetAddSql(edbInfoIdStr, edbCode, needDayStr, timestampStr, valStr)
					isAdd = true
				} else {
					//如果对应的值不匹配
					if tmpExistData.Value != valStr {
						err = ModifyEdbDataById(source, subSource, tmpExistData.EdbDataId, valStr)
						if err != nil {
							return
						}
					}
				}

			}
			existMap[existKey] = needDayStr
		}
		existKey := edbCode + currentItem.DataTime
		if _, ok := existMap[existKey]; !ok {
			currentDate, _ := time.ParseInLocation(utils.FormatDate, currentItem.DataTime, time.Local)
			timestamp := currentDate.UnixNano() / 1e6
			timestampStr := fmt.Sprintf("%d", timestamp)
			valStr := decimal.NewFromFloat(currentItem.Value).String()
			if fromEdbInfo.LatestDate == currentItem.DataTime {
				latestValue = currentItem.Value
			}
			tmpExistData, ok2 := existDataMap[currentItem.DataTime]
			if !ok2 {
				addSql += GetAddSql(edbInfoIdStr, edbCode, currentItem.DataTime, timestampStr, valStr)
				isAdd = true
			} else {
				//如果对应的值不匹配
				if tmpExistData.Value != valStr {
					err = ModifyEdbDataById(source, subSource, tmpExistData.EdbDataId, valStr)
					if err != nil {
						return
					}
				}
			}

		}
		existMap[existKey] = currentItem.DataTime
	}

	for k, _ := range removeDateMap {
		removeDateList = append(removeDateList, k)
	}
	// 删除不需要的指标数据
	if len(removeDateList) > 0 {
		//如果拼接指标变更了，那么需要删除所有的指标数据
		tableName := GetEdbDataTableName(source, subSource)
		sql := fmt.Sprintf(` DELETE FROM %s WHERE edb_info_id = ? and data_time in (`+utils.GetOrmInReplace(len(removeDateList))+`) `, tableName)
		err = to.Exec(sql, edbInfoId, removeDateList).Error
		if err != nil {
			err = fmt.Errorf("删除不存在的升频指标数据失败,Err:" + err.Error())
			return
		}
	}

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		err = to.Exec(addSql).Error
	}
	return
}
